@@ -8,9 +8,12 @@ import asyncio
 import json
 import os
 import random
+import re
 import ssl
+import sys
 import uuid
 from enum import Enum
+from pathlib import Path
 from typing import Generator
 from typing import Literal
 from typing import Optional
@@ -23,6 +26,7 @@ from prompt_toolkit import PromptSession
 from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
 from prompt_toolkit.completion import WordCompleter
 from prompt_toolkit.history import InMemoryHistory
+from prompt_toolkit.key_binding import KeyBindings
 from rich.live import Live
 from rich.markdown import Markdown
 
@@ -90,9 +94,9 @@ class NotAllowedToAccess(Exception):
 
 
 class ConversationStyle(Enum):
-    creative = "h3relaxedimg"
+    creative = "h3imaginative,clgalileo,gencontentv3"
     balanced = "galileo"
-    precise = "h3precise"
+    precise = "h3precise,clgalileo"
 
 
 CONVERSATION_STYLE_TYPE = Optional[
@@ -204,8 +208,7 @@ class Conversation:
 
     def __init__(
         self,
-        cookiePath: str = "",
-        cookies: dict | None = None,
+        cookies: dict,
         proxy: str | None = None,
     ) -> None:
         self.struct: dict = {
@@ -214,21 +217,23 @@ class Conversation:
             "conversationSignature": None,
             "result": {"value": "Success", "message": None},
         }
+        self.proxy = proxy
+        proxy = (
+            proxy
+            or os.environ.get("all_proxy")
+            or os.environ.get("ALL_PROXY")
+            or os.environ.get("https_proxy")
+            or os.environ.get("HTTPS_PROXY")
+            or None
+        )
+        if proxy is not None and proxy.startswith("socks5h://"):
+            proxy = "socks5://" + proxy[len("socks5h://") :]
         self.session = httpx.Client(
             proxies=proxy,
             timeout=30,
             headers=HEADERS_INIT_CONVER,
         )
-        if cookies is not None:
-            cookie_file = cookies
-        else:
-            f = (
-                open(cookiePath, encoding="utf8").read()
-                if cookiePath
-                else open(os.environ.get("COOKIE_FILE"), encoding="utf-8").read()
-            )
-            cookie_file = json.loads(f)
-        for cookie in cookie_file:
+        for cookie in cookies:
             self.session.cookies.set(cookie["name"], cookie["value"])
 
         # Send GET request
@@ -276,6 +281,7 @@ class ChatHub:
         prompt: str,
         wss_link: str,
         conversation_style: CONVERSATION_STYLE_TYPE = None,
+        raw: bool = False,
     ) -> Generator[str, None, None]:
         """
         Ask a question to the bot
@@ -301,7 +307,9 @@ class ChatHub:
                 if obj is None or not obj:
                     continue
                 response = json.loads(obj)
-                if response.get("type") == 1 and response["arguments"][0].get(
+                if response.get("type") != 2 and raw == True:
+                    yield False, response
+                elif response.get("type") == 1 and response["arguments"][0].get(
                     "messages",
                 ):
                     resp_txt = response["arguments"][0]["messages"][0]["adaptiveCards"][
@@ -331,15 +339,21 @@ class Chatbot:
 
     def __init__(
         self,
-        cookiePath: str = "",
-        cookies: dict | None = None,
+        cookies: dict = {},
         proxy: str | None = None,
+        cookiePath: str = None,
     ) -> None:
-        self.cookiePath: str = cookiePath
-        self.cookies: dict | None = cookies
+        if cookiePath is not None:
+            try:
+                with open(cookiePath, encoding="utf-8") as f:
+                    self.cookies = json.load(f)
+            except FileNotFoundError as e:
+                raise FileNotFoundError("Cookie file not found") from e
+        else:
+            self.cookies = cookies
         self.proxy: str | None = proxy
         self.chat_hub: ChatHub = ChatHub(
-            Conversation(self.cookiePath, self.cookies, self.proxy),
+            Conversation(self.cookies, self.proxy),
         )
 
     async def ask(
@@ -366,6 +380,7 @@ class Chatbot:
         prompt: str,
         wss_link: str = "wss://sydney.bing.com/sydney/ChatHub",
         conversation_style: CONVERSATION_STYLE_TYPE = None,
+        raw: bool = False,
     ) -> Generator[str, None, None]:
         """
         Ask a question to the bot
@@ -374,6 +389,7 @@ class Chatbot:
             prompt=prompt,
             conversation_style=conversation_style,
             wss_link=wss_link,
+            raw=raw,
         ):
             yield response
 
@@ -388,7 +404,7 @@ class Chatbot:
         Reset the conversation
         """
         await self.close()
-        self.chat_hub = ChatHub(Conversation(self.cookiePath, self.cookies))
+        self.chat_hub = ChatHub(Conversation(self.cookies))
 
 
 async def get_input_async(
@@ -406,22 +422,52 @@ async def get_input_async(
 
 
 def create_session() -> PromptSession:
-    return PromptSession(history=InMemoryHistory())
+    kb = KeyBindings()
+
+    @kb.add("enter")
+    def _(event):
+        buffer_text = event.current_buffer.text
+        if buffer_text.startswith("!"):
+            event.current_buffer.validate_and_handle()
+        else:
+            event.current_buffer.insert_text("\n")
+
+    @kb.add("escape")
+    def _(event):
+        if event.current_buffer.complete_state:
+            # event.current_buffer.cancel_completion()
+            event.current_buffer.text = ""
+
+    return PromptSession(key_bindings=kb, history=InMemoryHistory())
 
 
-async def main() -> None:
+def create_completer(commands: list, pattern_str: str = "$"):
+    return WordCompleter(words=commands, pattern=re.compile(pattern_str))
+
+
+async def async_main(args: argparse.Namespace) -> None:
     """
     Main function
     """
     print("Initializing...")
     print("Enter `alt+enter` or `escape+enter` to send a message")
-    bot = Chatbot(proxy=args.proxy)
+    bot = Chatbot(proxy=args.proxy, cookies=args.cookies)
     session = create_session()
+    completer = create_completer(["!help", "!exit", "!reset"])
+    initial_prompt = args.prompt
+
     while True:
         print("\nYou:")
-        question = (
-            input() if args.enter_once else await get_input_async(session=session)
-        )
+        if initial_prompt:
+            question = initial_prompt
+            print(question)
+            initial_prompt = None
+        else:
+            question = (
+                input()
+                if args.enter_once
+                else await get_input_async(session=session, completer=completer)
+            )
         print()
         if question == "!exit":
             break
@@ -478,7 +524,7 @@ async def main() -> None:
     await bot.close()
 
 
-if __name__ == "__main__":
+def main() -> None:
     print(
         """
         EdgeGPT - A demo of reverse engineering the Bing GPT chatbot
@@ -513,17 +559,32 @@ if __name__ == "__main__":
     parser.add_argument(
         "--cookie-file",
         type=str,
-        default="cookies.json",
+        default=os.environ.get("COOKIE_FILE", ""),
         required=False,
-        help="needed if environment variable COOKIE_FILE is not set",
+        help="Cookie file used for authentication (defaults to COOKIE_FILE environment variable)",
+    )
+    parser.add_argument(
+        "--prompt",
+        type=str,
+        default="",
+        required=False,
+        help="prompt to start with",
     )
     args = parser.parse_args()
-    if os.path.exists(args.cookie_file):
-        os.environ["COOKIE_FILE"] = args.cookie_file
-    else:
+    if args.cookie_file == "":
         parser.print_help()
         parser.exit(
             1,
-            "ERROR: use --cookied-file or set environemnt variable COOKIE_FILE",
+            "ERROR: use --cookie-file or set the COOKIE_FILE environment variable",
         )
-    asyncio.run(main())
+    try:
+        args.cookies = json.loads(Path(args.cookie_file).read_text(encoding="utf-8"))
+    except OSError as exc:
+        print(f"Could not open cookie file: {exc}", file=sys.stderr)
+        sys.exit(1)
+
+    asyncio.run(async_main(args))
+
+
+if __name__ == "__main__":
+    main()
